@@ -18,7 +18,7 @@ from ipython_genutils.importstring import import_item
 from .localinterfaces import is_local_ip, local_ips
 from traitlets import (
     Any, Float, Instance, Unicode, List, Bool, Type, DottedObjectName,
-    default, observe
+    default, observe, observe_compat
 )
 from jupyter_client import (
     launch_kernel,
@@ -63,6 +63,8 @@ class KernelManager(ConnectionFileMixin):
     def _kernel_spec_manager_default(self):
         return kernelspec.KernelSpecManager(data_dir=self.data_dir)
 
+    @observe('kernel_spec_manager')
+    @observe_compat
     def _kernel_spec_manager_changed(self):
         self._kernel_spec = None
 
@@ -191,7 +193,12 @@ class KernelManager(ConnectionFileMixin):
             # but it should be.
             cmd[0] = sys.executable
 
-        ns = dict(connection_file=self.connection_file,
+        # Make sure to use the realpath for the connection_file
+        # On windows, when running with the store python, the connection_file path
+        # is not usable by non python kernels because the path is being rerouted when
+        # inside of a store app.
+        # See this bug here: https://bugs.python.org/issue41196
+        ns = dict(connection_file=os.path.realpath(self.connection_file),
                   prefix=sys.prefix,
                  )
 
@@ -263,10 +270,8 @@ class KernelManager(ConnectionFileMixin):
             # If kernel_cmd has been set manually, don't refer to a kernel spec.
             # Environment variables from kernel spec are added to os.environ.
             env.update(self._get_env_substitutions(self.kernel_spec.env, env))
-        elif self.extra_env:
-            env.update(self._get_env_substitutions(self.extra_env, env))
-        kw['env'] = env
 
+        kw['env'] = env
         return kernel_cmd, kw
 
     def _get_env_substitutions(self, templated_env, substitution_values):
@@ -388,10 +393,23 @@ class KernelManager(ConnectionFileMixin):
             # most 1s, checking every 0.1s.
             self.finish_shutdown()
 
-        from . import __version__
-        from distutils.version import LooseVersion
+        # In 6.1.5, a new method, cleanup_resources(), was introduced to address
+        # a leak issue (https://github.com/jupyter/jupyter_client/pull/548) and
+        # replaced the existing cleanup() method.  However, that method introduction
+        # breaks subclass implementations that override cleanup() since it would
+        # circumvent cleanup() functionality implemented in subclasses.
+        # By detecting if the current instance overrides cleanup(), we can determine
+        # if the deprecated path of calling cleanup() should be performed - which avoids
+        # unnecessary deprecation warnings in a majority of configurations in which
+        # subclassed KernelManager instances are not in use.
+        # Note: because subclasses may have already implemented cleanup_resources()
+        # but need to support older jupyter_clients, we should only take the deprecated
+        # path if cleanup() is overridden but cleanup_resources() is not.
 
-        if LooseVersion(__version__) < LooseVersion('6.2'):
+        overrides_cleanup = type(self).cleanup is not KernelManager.cleanup
+        overrides_cleanup_resources = type(self).cleanup_resources is not KernelManager.cleanup_resources
+
+        if overrides_cleanup and not overrides_cleanup_resources:
             self.cleanup(connection_file=not restart)
         else:
             self.cleanup_resources(restart=restart)
@@ -446,7 +464,6 @@ class KernelManager(ConnectionFileMixin):
         This is a private method, callers should use shutdown_kernel(now=True).
         """
         if self.has_kernel:
-
             # Signal the kernel to terminate (sends SIGKILL on Unix and calls
             # TerminateProcess() on Win32).
             try:
@@ -470,8 +487,6 @@ class KernelManager(ConnectionFileMixin):
             # Block until the kernel terminates.
             self.kernel.wait()
             self.kernel = None
-        else:
-            raise RuntimeError("Cannot kill kernel. No kernel is running!")
 
     def interrupt_kernel(self):
         """Interrupts the kernel by sending it a signal.
@@ -609,10 +624,11 @@ class AsyncKernelManager(KernelManager):
             # most 1s, checking every 0.1s.
             await self.finish_shutdown()
 
-        from . import __version__
-        from distutils.version import LooseVersion
+        # See comment in KernelManager.shutdown_kernel().
+        overrides_cleanup = type(self).cleanup is not AsyncKernelManager.cleanup
+        overrides_cleanup_resources = type(self).cleanup_resources is not AsyncKernelManager.cleanup_resources
 
-        if LooseVersion(__version__) < LooseVersion('6.2'):
+        if overrides_cleanup and not overrides_cleanup_resources:
             self.cleanup(connection_file=not restart)
         else:
             self.cleanup_resources(restart=restart)
@@ -695,8 +711,6 @@ class AsyncKernelManager(KernelManager):
                 if self.kernel is not None:
                     self.kernel.wait()
             self.kernel = None
-        else:
-            raise RuntimeError("Cannot kill kernel. No kernel is running!")
 
     async def interrupt_kernel(self):
         """Interrupts the kernel by sending it a signal.
